@@ -1,6 +1,5 @@
 import math
 
-from robotpy_toolkit_7407.unum import Unum
 from wpimath.geometry import Rotation2d, Pose2d, Translation2d
 from wpimath.kinematics import SwerveDrive4Odometry, SwerveDrive4Kinematics, SwerveModuleState, ChassisSpeeds, SwerveModulePosition
 from wpimath.estimator import SwerveDrive4PoseEstimator
@@ -67,6 +66,15 @@ class SwerveNode:
         """
         ...
 
+    def get_node_position(self) -> SwerveModulePosition:
+        """
+        Get the position of the swerve node. Must be overridden. Must return SwerveModulePosition.
+
+        Returns:
+            SwerveModulePosition: position of the swerve node
+        """
+        ...
+
     # 0 degrees is facing right | "ethan is our FRC lord and saviour" - sid
     def _set_angle(self, target_angle: radians, initial_angle: radians):
         target_sensor_angle, flipped, flip_sensor_offset = SwerveNode._resolve_angles(target_angle, initial_angle)
@@ -178,22 +186,10 @@ class SwerveDrivetrain(Subsystem):
         )
 
         self.swerve_positions = (
-            SwerveModulePosition(
-                distance=((self.track_width * .5)**2 + (.5 * self.track_width)**2)**.5,
-                angle=Rotation2d(math.atan2(-.5 * self.track_width, -.5 * self.track_width))
-            ),
-            SwerveModulePosition(
-                distance=((self.track_width * .5)**2 + (.5 * self.track_width)**2)**.5,
-                angle=Rotation2d(math.atan2(-.5 * self.track_width, .5 * self.track_width)),
-            ),
-            SwerveModulePosition(
-                distance=((self.track_width * .5)**2 + (.5 * self.track_width)**2)**.5,
-                angle=Rotation2d(math.atan2(.5 * self.track_width, -.5 * self.track_width)),
-            ),
-            SwerveModulePosition(
-                distance=((self.track_width * .5)**2 + (.5 * self.track_width)**2)**.5,
-                angle=Rotation2d(math.atan2(.5 * self.track_width, .5 * self.track_width)),
-            )
+            self.n_00.get_node_position(),
+            self.n_01.get_node_position(),
+            self.n_10.get_node_position(),
+            self.n_11.get_node_position()
         )
 
         self.odometry = SwerveDrive4Odometry(
@@ -278,7 +274,15 @@ class SwerveDrivetrain(Subsystem):
             self.swerve_positions[2],
             self.swerve_positions[3]
         )
-        self.odometry_estimator.update(Rotation2d(self.gyro.get_robot_heading()), *self.swerve_positions)
+        self.odometry_estimator.update(
+            Rotation2d(self.gyro.get_robot_heading()),
+            (
+                self.swerve_positions[0],
+                self.swerve_positions[1],
+                self.swerve_positions[2],
+                self.swerve_positions[3]
+            )
+        )
         self.chassis_speeds = self.kinematics.toChassisSpeeds(*module_states)
 
     def stop(self):
